/*********************************************************
 * Copyright (C) 2019-2020 VMware, Inc. All rights reserved.
 *********************************************************/

/*
 * vmwareHzTypes.h
 *
 *      Definitions of common types used in SDK.
 */

#ifndef VMW_HORIZON_SDK_TYPES_H
#define VMW_HORIZON_SDK_TYPES_H


namespace vmware {
namespace horizon {
namespace client {


using HzWindowHandle = void *;
using HzWindowRectPtr = void *;


// The protocol used to connect to agent.
enum class HzSessionProtocol {
   UNKNOWN = -1,
   BLAST = 0,
   PCOIP = 1,
};


// The certificate check level when doing SSL handshake with agent.
enum class HzSecurityLevel {
   NONE = 0,
   WARN = 1,
   FULL = 2,
};


// The protocol connection state.
enum class HzSessionConnectionState {
   UNSET = -1,
   DISCONNECTED = 0,
   PENDING = 1,
   CONNECTED = 2,
};


// The grab/ungrab state in the remote desktop window.
enum class HzGrabState {
   UNGRAB_HARD = 0,
   UNGRAB_SOFT = 1,
   GRAB = 2,
   GRAB_MOTION = 3,
};


// The reason for connection state change.
enum class HzSessionConnectionStateReason {
   // Invalid, or unset.
   CONNECTION_INVALID = -1,

   // The connection to the remote computer has succeeded.
   CONNECTION_SUCCESS = 0,

   //The connection to the remote computer failed due to a generic failure.
   CONNECTION_FAILURE = 1,

   // The connection to the remote computer failed due to timeout.
   CONNECTION_TIMEOUT = 2,

   // The connection to the remote computer has been rejected.
   CONNECTION_REJECTED = 3,

   // The connection to the remote computer failed due to a network failure.
   NETWORK_FAILURE = 4,

   // The connection to the remote computer has been lost.
   CONNECTION_LOST = 5,

   /*
    * The connection to the remote computer ended due to the server initiated
    * the disconnect.
    */
   SERVER_DISCONNECTED = 6,

   // The connection to the remote computer failed due to a generic server error.
   SERVER_ERROR = 7,

   /*
    * The connection to the remote computer failed due to client side memory is
    * not sufficient for the required display configuration.
    */
   DISPLAY_NOT_ENOUGH_MEM = 8,

   // The connection to the remote computer ended due to unspecified reason.
   RESULT_UNSPECIFIED = 9,

   // The connection timed out thus disconnected by the server.
   SERVER_DISCONNECTED_EXPIRED = 10,

   /*
    * The protocol connection was disconnected due to user manually initiated to
    * disconnect or logout from the server.
    */
   SERVER_DISCONNECTED_MANUAL_LOGOUT = 11,

   /*
    * The protocol connection was disconnected due to the administrator manually
    * disconnected the session from the connection server admin console.
    */
   SERVER_DISCONNECTED_ADMIN_MANUAL = 12,

   // The remote desktop initiated the disconnect but it will reconnect later.
   SERVER_DISCONNECTED_RECONNECT = 13,

   // The connection failed due to an incompatible version of SVGA drivers.
   SERVER_SVGA_DRIVER_INCOMPATIBLE = 14,

   // Tera 1 host card user disconnected the connection.
   TERA_DISCONNECT_HOST_DRIVER_MANUAL_USER_DISCONNECT = 15,

   // The disconnection was caused by the Tera 1 host card driver version mismatch.
   TERA_DISCONNECT_HOST_DRIVER_INCOMPATIBLE_DRIVER_VERSION = 16,

   /*
    * The connection failed due to a common encryption protocol is not available.
    * Need to contact your system administrator for more information.
    */
   ENCRYPTION_MISMATCH = 17,
   /*
    * The connection to the remote computer has been closed. If you are connecting
    * through a Security Server, please ensure that AES encryption is enabled on
    * the client. Need to contact your system administrator for more information.
    */
   SERVER_DISCONNECTED_AES_DISABLED_LOCALLY = 18,
   // The remote desktop's certificate is invalid.
   CERTIFICATE_REJECTED_INVALID = 19,

   /*
    * The server certificate is the legacy PCoIP cert.
    * It would only be connected when the security level is set as
    * HzSecurityLevel::NONE or HzSecurityLevel::WARN.
    */
   CERTIFICATE_REJECTED_LEGACY = 20,

   /*
    * The certificate of the remote desktop is weak. Under such case, when the
    * security level is set as HzSecurityLevel::NONE or HzSecurityLevel::WARN,
    * it is allowed to connect the remote desktop. It is not allowed when the
    * security level is set as HzSecurityLevel::FULL.
    */
   SECURITY_FAILED_AT_FULL = 21,

   // The connection failed due to the FQDN of the remote desktop is not resolvable.
   HOSTNAME_NOT_RESOLVABLE = 22,

   // The client user has initiated to disconnect from the remote desktop.
   USER_INIT_DISCONNECT = 23,

   // The client user has initiated to disconnect and log off from the remote desktop.
   USER_INIT_DISCONNECTANDLOGOFF = 24,

   // The client user has initiated to reset the remote desktop.
   USER_INIT_RESET = 25,

   // The protocol layer asked the client to disconnect the session.
   CLIENT_SHOULD_DISCONNECT = 26,

   // Generic session logout (regardless of the initiator as the client or the agent side).
   GENERIC_LOGOUT = 27,

   /*
    * In a generic scenario where remote application was restarted or shut down but the
    * cause was not known exactly.
    * For example, on some systems, it doesn't tell us reliably that a system
    * restart/shutdown is happening that cause this app restart/shutdown. All the
    * application knows is that it was asked to exit (or be killed).
    * This case will cover such situation.
    */
   GENERIC_APP_SHUTDOWN_RESTART = 28,

  /*
    * The connection state was due to the network failure.
    * But the network continuity has been enabled thus the system will try to recover
    * the connection.
    */
   NETWORK_FAILURE_WITH_CONTINUITY = 29,

   // The shadow session has been ended.
   SERVER_SHADOW_SESSION_ENDED = 30,
};


// The structure containing the hot key to be sent to the remote session window.
typedef struct {
   unsigned int keyScancode;
   unsigned int modifiers;
} HzHotKey;


} // namespace client
} // namespace horizon
} // namespace vmware

#endif // VMW_HORIZON_SDK_TYPES_H
